@@ -28,9 +28,17 @@
 #include "ggv-window.h"
 #include "ggv-file-sel.h"
 #include "ggvutils.h"
+#include "ggv-recent.h"
+#include "recent-files/egg-recent-view.h"
+#include "recent-files/egg-recent-view-bonobo.h"
+#include "recent-files/egg-recent-view-gtk.h"
+#include "recent-files/egg-recent-model.h"
 
 #define GGV_POSTSCRIPT_VIEWER_CONTROL_IID "OAFIID:GNOME_GGV_Control"
 
+#define GGV_WATCH_TIMEOUT 2
+#define GGV_WATCH_INTERVAL 1000
+
 static BonoboWindowClass *parent_class;
 
 static GList *window_list = NULL;
@@ -40,6 +48,24 @@ enum {
         TARGET_URI_LIST,
 };
 
+static void
+set_file_cmds_sensitivity(GgvWindow *win, gboolean sens)
+{
+        if(NULL == win->uic)
+                return;
+
+        bonobo_ui_component_set_prop(win->uic,
+                                     "/commands/FileReload",
+                                     "sensitive",
+                                     sens?"1":"0",
+                                     NULL);
+        bonobo_ui_component_set_prop(win->uic,
+                                     "/commands/FileClose",
+                                     "sensitive",
+                                     sens?"1":"0",
+                                     NULL);
+}
+
 static void
 sync_settings_menu_items(GgvWindow *win)
 {
@@ -76,51 +102,6 @@ sync_settings_popup_items(GgvWindow *win)
 	bonobo_ui_component_thaw(win->popup_uic, NULL);
 }
 
-void
-sync_toolbar_style(GgvWindow *win)
-{
-        const gchar *tbstyle;
-        gchar *gnome_tbstyle;
-
-        switch(ggv_toolbar_style) {
-        case GGV_TOOLBAR_STYLE_DEFAULT:
-                gnome_tbstyle = gconf_client_get_string(ggv_prefs_gconf_client(),
-                                                        "/desktop/gnome/interface/toolbar_style",
-                                                        NULL);
-                if(gnome_tbstyle) {
-                        if(!strcmp(gnome_tbstyle, "icons"))
-                                tbstyle = "icon";
-                        else if(!strcmp(gnome_tbstyle, "both"))
-                                tbstyle = "both";
-                        else if(!strcmp(gnome_tbstyle, "text"))
-                                tbstyle = "text";
-                        else if(!strcmp(gnome_tbstyle, "both_horiz"))
-                                tbstyle = "both_horiz";
-                        else
-                                tbstyle = "both";
-                        g_free(gnome_tbstyle);
-                }
-                else
-                        tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_BOTH:
-                tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_ICONS:
-                tbstyle = "icon";
-                break;
-        case GGV_TOOLBAR_STYLE_TEXT:
-                tbstyle = "text";
-                break;
-        default:
-                tbstyle = NULL;
-                break;
-        }
-        if(tbstyle)
-                bonobo_ui_component_set_prop(win->uic, "/Toolbar", "look",
-                                             tbstyle, NULL);
-}
-
 static void
 sync_fullscreen_items(GgvWindow *win)
 {
@@ -134,6 +115,55 @@ sync_fullscreen_items(GgvWindow *win)
         }
 }
 
+static void
+ggv_window_remove_control(GgvWindow *win)
+{
+        CORBA_Environment ev;
+        
+        CORBA_exception_init(&ev);
+
+        if(win->ctlframe != NULL && win->control != CORBA_OBJECT_NIL) {
+                bonobo_control_frame_control_deactivate(win->ctlframe);
+                bonobo_control_frame_bind_to_control(win->ctlframe, NULL, &ev);
+                if(BONOBO_EX(&ev)) {
+                        g_warning("Could not bind to NIL: '%s'",
+                                  bonobo_exception_get_text(&ev));
+                        CORBA_exception_free(&ev);
+                        return;
+                }
+        }
+        if(win->control != CORBA_OBJECT_NIL) {
+                bonobo_object_release_unref(win->control, NULL);
+                win->control = CORBA_OBJECT_NIL;
+        }
+        if(win->popup_uic) {
+                bonobo_object_unref(win->popup_uic);
+                win->popup_uic = NULL;
+        }
+        if(win->pb) {
+                bonobo_object_release_unref(win->pb, NULL);
+                win->pb = CORBA_OBJECT_NIL;
+        }
+#if 0
+        if(win->view) {
+                gtk_container_remove(GTK_CONTAINER(win->hbox),
+                                     win->view);
+                gtk_widget_unref(win->view);
+                win->view = NULL;
+        }
+#endif
+        if(win->filename) {
+                g_free(win->filename);
+                win->filename = NULL;
+        }
+        if(0 != win->timer_tag) {
+                gtk_timeout_remove(win->timer_tag);
+                win->timer_tag = 0;
+        }
+        set_file_cmds_sensitivity(win, FALSE);
+        CORBA_exception_free(&ev);                
+}
+
 /* utility functions to raise window above all others! */
 #define XA_WIN_LAYER         "_WIN_LAYER"
 typedef enum {
@@ -351,22 +381,16 @@ control_property_changed_handler(BonoboListener    *listener,
                 }
         }
         else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileReload",
-                                             "sensitive",
-                                             (BONOBO_ARG_GET_LONG(any) > 0)?"1":"0",
-                                             NULL);
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileClose",
-                                             "sensitive",
-                                             (BONOBO_ARG_GET_LONG(any) > 0)?"1":"0",
-                                             NULL);
+                set_file_cmds_sensitivity(win, (BONOBO_ARG_GET_LONG(any) > 0));
         }
 }
 
 static void
 verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
+        EggRecentItem *item;
+        EggRecentModel *recent;
+
         GgvWindow *win = GGV_WINDOW(data);
         gchar *fname, *escaped_fname;
 
@@ -385,6 +409,12 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
                         gtk_dialog_run(GTK_DIALOG(dlg));
                         gtk_widget_destroy(dlg);
                 }
+		recent = ggv_recent_get_model ();
+		item = egg_recent_item_new_from_uri (fname);
+		egg_recent_item_add_group (item, "ggv");
+		egg_recent_model_add_full (recent, item);
+		egg_recent_item_unref (item);
+
                 g_free(fname);
         }
 }
@@ -433,6 +463,7 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
                         bonobo_object_release_unref(ps_view, &ev);
                 }
                 CORBA_exception_free(&ev);                
+                ggv_window_set_watch_file(win, FALSE);
         }
 }
 
@@ -545,8 +576,9 @@ static void
 verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	static GtkWidget *about = NULL;
+	static gboolean translated = FALSE;
 	static const char *authors[] = {
-                "Jaka Mocnik (current maintainer)",
+                N_("Jaka Mocnik (current maintainer)"),
                 "Jonathan Blandford",
                 "Daniel M. German",
                 "Satyajit Kanungo",
@@ -556,19 +588,24 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
                 "Johannes Plass",
 		"Istvan Szekeres",
                 "Tim Theisen",
-                "And many more...",
+                N_("And many more..."),
  		NULL
 	};
-
 	static const char *documenters[] = {
+		"Sun Microsystems <gdocteam@sun.com>",
 		NULL
 	};
-
-	/* Translator Credits */
 	char *translator_credits = _("translator_credits");
-
         static GdkPixbuf *logo = NULL;
 
+	if (!translated) {
+		int i;
+
+		for (i = 0; authors[i] != NULL; i++)
+		    authors[i] = _(authors[i]);
+		translated = TRUE;
+	}
+
         if(!logo)
                 logo = gdk_pixbuf_new_from_file(GNOMEICONDIR
                                                 "/ggv/ggv-splash.png",
@@ -687,6 +724,7 @@ static void
 ggv_window_destroy(GtkObject *object)
 {
         GgvWindow *win;
+        gint i;
 
         g_return_if_fail(object != NULL);
         g_return_if_fail(GGV_IS_WINDOW(object));
@@ -717,6 +755,10 @@ ggv_window_destroy(GtkObject *object)
                 g_free(win->filename);
                 win->filename = NULL;
         }
+        if(0 != win->timer_tag) {
+                gtk_timeout_remove(win->timer_tag);
+                win->timer_tag = 0;
+        }
 
         window_list = g_list_remove(window_list, win);
 
@@ -738,15 +780,7 @@ static void
 prefs_changed(GConfClient* client, guint cnxn_id,
               GConfEntry *entry, gpointer user_data)
 {
-        if(!strcmp(entry->key, "/desktop/gnome/interface/toolbar_style") &&
-           ggv_toolbar_style == GGV_TOOLBAR_STYLE_DEFAULT) {
-                sync_toolbar_style(GGV_WINDOW(user_data));
-        }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/toolbarstyle")) {
-                ggv_toolbar_style = gconf_value_get_int(entry->value);
-                sync_toolbar_style(GGV_WINDOW(user_data));
-        }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
+        if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
                 GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_toolbar != state_f) {
@@ -817,6 +851,7 @@ static void
 ggv_window_init (GgvWindow *window)
 {
         window_list = g_list_prepend(window_list, window);
+        window->watch_doc = FALSE;
 }
 
 GType
@@ -846,6 +881,124 @@ ggv_window_get_type (void)
 	return ggv_window_type;
 }
 
+static void
+menu_position_under_widget (GtkMenu *menu, int *x, int *y,
+                            gboolean *push_in, gpointer user_data)
+{
+        GtkWidget *w;
+        int screen_width, screen_height;
+        GtkRequisition requisition;
+                                                                                                                             
+        w = GTK_WIDGET (user_data);
+                                                                                                                             
+        gdk_window_get_origin (w->window, x, y);
+        *x += w->allocation.x;
+        *y += w->allocation.y + w->allocation.height;
+                                                                                                                             
+        gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+                                                                                                                             
+        screen_width = gdk_screen_width ();
+        screen_height = gdk_screen_height ();
+                                                                                                                             
+        *x = CLAMP (*x, 0, MAX (0, screen_width - requisition.width));
+        *y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));
+}
+
+static gboolean
+open_button_pressed_cb (GtkWidget *widget,
+                        GdkEventButton *event,
+                        gpointer *user_data)
+{
+        GtkWidget *menu;
+                                                                                                                             
+        g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE); 
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE); 
+
+	menu = g_object_get_data (G_OBJECT (widget), "recent-menu");
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, menu_position_under_widget, widget,
+			event->button, event->time);
+ 
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+        return TRUE;
+}
+
+static gboolean
+open_button_key_pressed_cb (GtkWidget *widget,
+                            GdkEventKey *event,
+                            gpointer *user_data)
+{
+        if (event->keyval == GDK_space ||
+            event->keyval == GDK_KP_Space ||
+            event->keyval == GDK_Return ||
+            event->keyval == GDK_KP_Enter) {
+                open_button_pressed_cb (widget, NULL, user_data);
+        }
+                                                                                                                             
+        return FALSE;
+}
+
+static gboolean
+ggv_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
+{
+        GtkWidget *newwin;
+        GgvWindow *win = GGV_WINDOW (data);
+        gchar *uri_utf8;
+                                                                                                                             
+        uri_utf8 = egg_recent_item_get_uri_utf8 (item);
+                                                                                                                             
+        if(win->current_page < 0)
+                newwin = GTK_WIDGET (win);
+        else
+                newwin = NULL;
+                                                                                                                             
+        if (newwin == NULL) {
+                newwin = ggv_window_new ();
+                gtk_widget_show (GTK_WIDGET (newwin));
+        }
+                                                                                                                             
+        ggv_window_load (GGV_WINDOW (newwin), uri_utf8);
+        newwin = NULL;
+                                                                                                                             
+        return TRUE;
+}
+
+ggv_add_open_button (GgvWindow *win, BonoboUIComponent *ui_component,
+                     const gchar *path, const gchar *tooltip)
+{
+        GtkWidget *menu;
+        EggRecentViewGtk *view;
+        EggRecentModel *model;
+        GtkWidget *button;
+                                                                                                                             
+        button = gtk_toggle_button_new ();
+        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+                                                                                                                             
+        gtk_container_add (GTK_CONTAINER (button),
+                           gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT));
+                                                                                                                             
+        gtk_widget_show_all (GTK_WIDGET (button));
+                                                                                                                             
+        model = ggv_recent_get_model ();
+                                                                                                                             
+        menu = gtk_menu_new ();
+        gtk_widget_show (menu);
+        view = egg_recent_view_gtk_new (menu, NULL);
+        g_signal_connect (view, "activate",
+                          G_CALLBACK (ggv_file_open_recent), win);
+        egg_recent_view_gtk_show_icons (view, TRUE);
+        egg_recent_view_gtk_show_numbers (view, FALSE);
+        egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+        g_object_set_data (G_OBJECT (button), "recent-menu", menu);
+        g_signal_connect_object (button, "key_press_event",
+                                 G_CALLBACK (open_button_key_pressed_cb),
+                                 win, 0);
+        g_signal_connect_object (button, "button_press_event",
+                                 G_CALLBACK (open_button_pressed_cb),
+                                 win, 0);
+        bonobo_ui_component_widget_set (ui_component, path, GTK_WIDGET (button),
+                                        NULL);
+}
+
 GtkWidget *
 ggv_window_new(void)
 {
@@ -855,6 +1008,8 @@ ggv_window_new(void)
         CORBA_Environment ev;
         Bonobo_Control control;
         GtkWidget *widget;
+        EggRecentViewBonobo *view;
+        EggRecentModel *model;
         Bonobo_PropertyBag pb;
         gchar *mask;
 
@@ -916,8 +1071,12 @@ ggv_window_new(void)
         win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
+
+
 	bonobo_ui_util_set_ui(uic, DATADIR, "ggv-ui.xml", "GGV", NULL);
-        sync_toolbar_style(win);
+
+        ggv_add_open_button (win, uic, "/Toolbar/TBOpenMenu", _("Open a file."));
+
 	bonobo_ui_component_add_verb_list_with_data(uic, ggv_app_verbs, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowMenus",
                                          listener_SettingsShow, win);
@@ -929,8 +1088,17 @@ ggv_window_new(void)
                                          listener_SettingsShow, win);
 	bonobo_ui_component_add_listener(uic, "ViewFullscreen",
                                          listener_ViewFullscreen, win);
+
         sync_settings_menu_items(win);
 
+        model = ggv_recent_get_model ();
+        view = egg_recent_view_bonobo_new (uic, "/menu/File/Recents");
+        egg_recent_view_bonobo_show_icons (EGG_RECENT_VIEW_BONOBO (view), FALSE);
+        egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+                                                                                                                             
+        g_signal_connect (G_OBJECT (view), "activate",
+                          G_CALLBACK (ggv_file_open_recent), win);
+                                                                                                                             
         if(!win->show_toolbar)
                 bonobo_ui_component_set_prop(uic, "/Toolbar", "hidden", "1", NULL);
         if(!win->show_menus)
@@ -1004,10 +1172,6 @@ ggv_window_new(void)
                              ggv_default_width, ggv_default_height);
         gtk_window_set_policy(GTK_WINDOW(win), TRUE, TRUE, FALSE);
 
-        gconf_client_notify_add(ggv_prefs_gconf_client(),
-                                "/desktop/gnome/interface/toolbar_style",
-                                (GConfClientNotifyFunc)prefs_changed,
-                                win, NULL, NULL);
         gconf_client_notify_add(ggv_prefs_gconf_client(),
                                 "/apps/ggv/layout",
                                 (GConfClientNotifyFunc)prefs_changed,
@@ -1022,11 +1186,22 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
         gchar *title, *unescaped, *utf8_title;
+        GnomeVFSFileInfo info;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
         g_return_val_if_fail(filename != NULL, FALSE);
 
+        ggv_window_set_watch_file(win, FALSE);
+
+	if (GNOME_VFS_OK == gnome_vfs_get_file_info(filename, &info, 
+                                                    GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+                                                    GNOME_VFS_FILE_INFO_FOLLOW_LINKS)) {
+                win->mtime = info.mtime;
+        }
+        else
+                return FALSE;
+
         CORBA_exception_init(&ev);
         pf = Bonobo_Unknown_queryInterface(win->control,
                                            "IDL:Bonobo/PersistFile:1.0", &ev);
@@ -1056,6 +1231,17 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         }
         CORBA_exception_free(&ev);
 
+        bonobo_ui_component_set_prop
+                (win->uic, "/commands/FileReload", "sensitive",
+                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
+                 NULL);
+        bonobo_ui_component_set_prop
+                (win->uic, "/commands/FileClose", "sensitive",
+                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
+                 NULL);
+
+        ggv_window_set_watch_file(win, ggv_watch_doc);
+
         return TRUE;
 }
 
@@ -1071,23 +1257,78 @@ ggv_window_close(GgvWindow *win)
                 bonobo_main_quit();
 }
 
-void
-ggv_window_apply_prefs(GgvWindow *win)
-{
-        if(win) {
-                sync_toolbar_style(win);
-        }
-        else {
-                GList *node = window_list;
-                while(node) {
-                        ggv_window_apply_prefs(GGV_WINDOW(node->data));
-                        node = node->next;
-                }
-        }
-}
-
 const GList *
 ggv_get_window_list()
 {
         return window_list;
 }
+
+/* Reload file if it was changed */
+static gint
+timer_callback (gpointer data)
+{
+        GgvWindow *win;
+        GnomeVFSFileInfo info;
+	struct stat sbuf;
+
+        win = GGV_WINDOW(data);
+
+        if(NULL == win->filename)
+                return FALSE;
+
+        /* if global prefs changed, change our local prefs */
+        if(!ggv_watch_doc) {
+                ggv_window_set_watch_file(win, FALSE);
+                return FALSE;
+        }
+
+        if(GNOME_VFS_OK != gnome_vfs_get_file_info(win->filename, &info, 0))
+                return TRUE;
+
+        /* reload file if necessary */
+	if ((win->current_page >= 0) &&
+	    (info.mtime != win->mtime) &&
+	    (time (NULL) - info.mtime >= GGV_WATCH_TIMEOUT) &&
+            (NULL != win->control)) {
+                GNOME_GGV_PostScriptView ps_view;
+                CORBA_Environment ev;
+
+                CORBA_exception_init(&ev);
+                ps_view = (GNOME_GGV_PostScriptView)
+                        Bonobo_Unknown_queryInterface(win->control,
+                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
+                                                      &ev);
+                if(!BONOBO_EX(&ev)) {
+                        GNOME_GGV_PostScriptView_reload(ps_view, &ev);
+                        if(BONOBO_EX(&ev))
+                                g_warning("Could not invoke reload method.");
+                        bonobo_object_release_unref(ps_view, &ev);
+                }
+                CORBA_exception_free(&ev);
+                win->mtime = info.mtime;
+        }
+
+        return TRUE;
+}
+
+void
+ggv_window_set_watch_file(GgvWindow *win, gboolean f)
+{
+        if(win->watch_doc == f)
+                return;
+
+        win->watch_doc = f;
+	if(win->watch_doc) {
+		if(win->timer_tag)
+                        gtk_timeout_remove(win->timer_tag);
+		win->timer_tag =
+                        gtk_timeout_add(GGV_WATCH_INTERVAL,
+                                        timer_callback, win);
+	}
+        else {
+		if(win->timer_tag) {
+                        gtk_timeout_remove(win->timer_tag);
+                        win->timer_tag = 0;
+                }
+        }
+}
